FRONTEND_CONNECTION_PATH = r"/interact"
INTERMITTENT_WAIT_TIMEOUT = 5  # seconds
DDD_DB_NAME = "ddds"
PROTOCOL_VERSION = "3.4"
DEPRECATED_PROTOCOL_VERSIONS = [2.0, "3.0", "3.1", "3.2", "3.3"]
SUPPORTED_PROTOCOL_VERSIONS = DEPRECATED_PROTOCOL_VERSIONS + [PROTOCOL_VERSION]

DEFAULT_PORT = 9090
